#include <deque>

/** This class implements an "array" indexed by the integers rather than the 
    natural numbers. It extends the deque STL container. */

using namespace std;

template <class T>
class bivector : public deque<T> {
 private:
  int start;
  int sz; /* variable storing the size */

  inline void push_front(const T &t)
    { deque<T>::push_front(t); sz++; start--; }

  inline void push_back(const T &t)
    { deque<T>::push_back(t); sz++; }

 public:
  bivector<T>():sz(0){}

  inline const T &operator[](int i) const
    { return deque<T>::operator[](i-start); }

  T &operator[](int i){
    if (empty()){
      push_back(T());
      start=i;
      return back();
    } 
    if (i<start){
      while (start>i)
	push_front(T());  
      return front();
    }
    if (i>=start+sz){
      while (i>=start+sz)
	push_back(T());
      return back();
    }
    return deque<T>::operator[](i-start);
  }

  inline int size() const { return sz; }

  /** The index of the first element */
  inline int begin_index() const 
    { return start; }
  /** The index of the last element stored plus one */
  inline int end_index() const 
    { return start+sz; }
};
